package it.uniroma2.flink;

import org.apache.flink.api.common.functions.FilterFunction;
import org.apache.flink.api.common.functions.MapFunction;
import org.apache.flink.streaming.api.datastream.DataStream;
import org.apache.flink.streaming.api.datastream.IterativeStream;
import org.apache.flink.streaming.api.environment.StreamExecutionEnvironment;

public class IterateOnStream {

    /*
     * We create a topology as follows:
     *
     *  someIntegers -(iteration)-> minusOne -> <> ->
     *                    ^_____________________|
     */
    public static void main(String[] args) throws Exception {

        final StreamExecutionEnvironment env = StreamExecutionEnvironment.getExecutionEnvironment();
        long threshold = 2L;

        DataStream<Long> someIntegers = env.fromSequence(0, 10);

        IterativeStream<Long> iteration = someIntegers.iterate();

        DataStream<Long> minusOne = iteration.map((MapFunction<Long, Long>) value -> value - 1);
        DataStream<Long> stillGreaterThanZero = minusOne.filter((FilterFunction<Long>) value -> value > threshold);
        iteration.closeWith(stillGreaterThanZero);

        DataStream<Long> result = minusOne.filter((FilterFunction<Long>) value -> value <= threshold);
        result.print();
        // Be careful, the result set also includes values: -1, 0, 1; they result from initial values 0, 1, 2 AND by
        // iteration construction that first decrement the values and then closes the loop.

        env.execute("Iterate on Stream");
    }

}
